@@ -16,8 +16,9 @@
 #include "wall-clock.h"
 #include "background-cache.h"
 
-#include <handy.h>
 #include <call-ui.h>
+#include <handy.h>
+#include <libfeedback.h>
 
 #include <glib/gi18n.h>
 #include <glib-unix.h>
@@ -130,6 +131,7 @@ main (int argc, char *argv[])
   gtk_init (&argc, &argv);
   hdy_init ();
   cui_init (TRUE);
+  lfb_init (PHOSH_APP_ID, NULL);
 
   g_unix_signal_add (SIGTERM, on_shutdown_signal, NULL);
   g_unix_signal_add (SIGINT, on_shutdown_signal, NULL);
