@@ -23,6 +23,7 @@
 #include "torch-info.h"
 #include "torch-manager.h"
 #include "vpn-manager.h"
+#include "bt-status-page.h"
 #include "wwan/wwan-manager.h"
 #include "notifications/notify-manager.h"
 #include "notifications/notification-frame.h"
@@ -72,8 +73,7 @@ typedef struct _PhoshSettings
   GtkWidget *media_player;
   PhoshAudioSettings *audio_settings;
 
-  GtkWidget *stack;
-  GtkWidget *status_page_stack;
+  GtkStack  *stack;
 
   /* The area with media widget, notifications */
   GtkWidget *box_bottom_half;
@@ -179,7 +179,7 @@ calc_drag_handle_offset (PhoshSettings *self)
   if (g_strcmp0 (stack_page, "status_page") != 0)
     goto out;
 
-  success = gtk_widget_translate_coordinates (self->stack, GTK_WIDGET (self),
+  success = gtk_widget_translate_coordinates (GTK_WIDGET (self->stack), GTK_WIDGET (self),
                                               0, 0, NULL, &stack_y);
 
   if (!success) {
@@ -187,7 +187,7 @@ calc_drag_handle_offset (PhoshSettings *self)
     goto out;
   }
 
-  stack_height = gtk_widget_get_allocated_height (self->stack);
+  stack_height = gtk_widget_get_allocated_height (GTK_WIDGET (self->stack));
   h = stack_y + stack_height;
 
   g_debug ("Calculating drag offset: stack_y = %d, stack_height = %d, height = %d",
@@ -211,14 +211,13 @@ on_size_allocate (PhoshSettings *self)
 }
 
 
-static gboolean
+static void
 delayed_update_drag_handle_offset (gpointer data)
 {
   PhoshSettings *self = PHOSH_SETTINGS (data);
 
   self->debounce_handle = 0;
   calc_drag_handle_offset (self);
-  return G_SOURCE_REMOVE;
 }
 
 
@@ -226,11 +225,35 @@ static void
 update_drag_handle_offset (PhoshSettings *self)
 {
   g_clear_handle_id (&self->debounce_handle, g_source_remove);
-  self->debounce_handle = g_timeout_add (200, delayed_update_drag_handle_offset, self);
+  self->debounce_handle = g_timeout_add_once (200, delayed_update_drag_handle_offset, self);
   g_source_set_name_by_id (self->debounce_handle, "[phosh] delayed_update_drag_handle_offset");
 }
 
 
+static void
+on_stack_visible_child_changed (PhoshSettings *self)
+{
+  static GtkWidget *last_status_page;
+  GtkWidget *child, *revealer;
+  gboolean reveal;
+
+  update_drag_handle_offset (self);
+
+  child = gtk_stack_get_visible_child (GTK_STACK (self->stack));
+  if (child == self->quick_settings && last_status_page) {
+    revealer = last_status_page;
+    reveal = FALSE;
+  } else {
+    g_return_if_fail (GTK_IS_REVEALER (child));
+    revealer = child;
+    reveal = TRUE;
+  }
+
+  gtk_revealer_set_reveal_child (GTK_REVEALER (revealer), reveal);
+  last_status_page = revealer;
+}
+
+
 static void
 close_settings_menu (PhoshSettings *self)
 {
@@ -369,8 +392,6 @@ wifi_setting_clicked_cb (PhoshSettings *self)
 static void
 wifi_setting_long_pressed_cb (PhoshSettings *self)
 {
-  GtkStack *stack = GTK_STACK (self->stack);
-  GtkStack *status_page_stack = GTK_STACK (self->status_page_stack);
   PhoshShell *shell = phosh_shell_get_default ();
   PhoshWifiManager *manager;
 
@@ -383,8 +404,7 @@ wifi_setting_long_pressed_cb (PhoshSettings *self)
   if (phosh_wifi_manager_get_enabled (manager))
     phosh_wifi_manager_request_scan (manager);
 
-  gtk_stack_set_visible_child_name (stack, "status_page");
-  gtk_stack_set_visible_child_name (status_page_stack, "wifi_status_page");
+  gtk_stack_set_visible_child_name (self->stack, "wifi_status_page");
 }
 
 static void
@@ -409,9 +429,11 @@ wwan_setting_long_pressed_cb (PhoshSettings *self)
   open_settings_panel (self, "wwan");
 }
 
+
 static void
-bt_setting_clicked_cb (PhoshSettings *self)
+on_toggle_bt_activated (GSimpleAction *action, GVariant *param, gpointer data)
 {
+  PhoshSettings *self = PHOSH_SETTINGS (data);
   PhoshShell *shell = phosh_shell_get_default ();
   PhoshBtManager *manager;
   gboolean enabled;
@@ -429,7 +451,10 @@ bt_setting_clicked_cb (PhoshSettings *self)
 static void
 bt_setting_long_pressed_cb (PhoshSettings *self)
 {
-  open_settings_panel (self, "bluetooth");
+  if (self->on_lockscreen)
+    return;
+
+  gtk_stack_set_visible_child_name (self->stack, "bt_status_page");
 }
 
 
@@ -674,14 +699,16 @@ on_notification_frames_items_changed (PhoshSettings *self,
 static void
 setup_brightness_range (PhoshSettings *self)
 {
+  gulong value_changed_handler_id;
+
   gtk_range_set_range (GTK_RANGE (self->scale_brightness), 0, 100);
   gtk_range_set_round_digits (GTK_RANGE (self->scale_brightness), 0);
   gtk_range_set_increments (GTK_RANGE (self->scale_brightness), 1, 10);
-  brightness_init (GTK_SCALE (self->scale_brightness));
-  g_signal_connect (self->scale_brightness,
+  value_changed_handler_id = g_signal_connect (self->scale_brightness,
                     "value-changed",
                     G_CALLBACK(brightness_value_changed_cb),
                     NULL);
+  brightness_init (GTK_SCALE (self->scale_brightness), value_changed_handler_id);
 }
 
 
@@ -795,6 +822,7 @@ phosh_settings_class_init (PhoshSettingsClass *klass)
   object_class->set_property = phosh_settings_set_property;
   object_class->get_property = phosh_settings_get_property;
 
+  g_type_ensure (PHOSH_TYPE_BT_STATUS_PAGE);
   g_type_ensure (PHOSH_TYPE_WIFI_STATUS_PAGE);
 
   gtk_widget_class_set_template_from_resource (widget_class,
@@ -844,12 +872,10 @@ phosh_settings_class_init (PhoshSettingsClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PhoshSettings, scale_brightness);
   gtk_widget_class_bind_template_child (widget_class, PhoshSettings, scale_torch);
   gtk_widget_class_bind_template_child (widget_class, PhoshSettings, stack);
-  gtk_widget_class_bind_template_child (widget_class, PhoshSettings, status_page_stack);
   gtk_widget_class_bind_template_child (widget_class, PhoshSettings, stack_notifications);
   gtk_widget_class_bind_template_child (widget_class, PhoshSettings, scrolled_window);
 
   gtk_widget_class_bind_template_callback (widget_class, battery_setting_clicked_cb);
-  gtk_widget_class_bind_template_callback (widget_class, bt_setting_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, bt_setting_long_pressed_cb);
   gtk_widget_class_bind_template_callback (widget_class, docked_setting_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, docked_setting_long_pressed_cb);
@@ -868,6 +894,7 @@ phosh_settings_class_init (PhoshSettingsClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, on_torch_scale_value_changed);
   gtk_widget_class_bind_template_callback (widget_class, on_vpn_setting_long_pressed);
   gtk_widget_class_bind_template_callback (widget_class, on_vpn_setting_clicked);
+  gtk_widget_class_bind_template_callback (widget_class, on_stack_visible_child_changed);
   gtk_widget_class_bind_template_callback (widget_class, update_drag_handle_offset);
 }
 
@@ -875,6 +902,7 @@ phosh_settings_class_init (PhoshSettingsClass *klass)
 static const GActionEntry entries[] = {
   { .name = "launch-panel", .activate = on_launch_panel_activated, .parameter_type = "s" },
   { .name = "close-status-page", .activate = on_close_status_page_activated },
+  { .name = "toggle-bt", .activate = on_toggle_bt_activated },
 };
 
 
